<?php

/**
 * @file
 * Callback methods for Global Payments Remote 3DSecure2 integration.
 */

/**
 * Build a settings form.
 */
function global_payments_administrator_terminal_commerce_payment_method_settings_form($settings = NULL) {
  $form = [];

  // TODO: learn if we need any other fields than what follows.
  $form['merchant_id'] = [
    '#type' => 'textfield',
    '#title' => t('Global Payments Merchant ID'),
    '#default_value' => $settings['merchant_id'] ?? '',
    '#size' => 16,
    '#required' => TRUE,
  ];

  $form['account_id'] = [
    '#type' => 'textfield',
    '#title' => t('Global Payments Account ID'),
    '#default_value' => $settings['account_id'] ?? '',
    '#size' => 16,
    '#required' => FALSE,
  ];

  $form['shared_secret'] = [
    '#type' => 'textfield',
    '#title' => t('Shared secret'),
    '#default_value' => $settings['shared_secret'] ?? '',
    '#size' => 16,
    '#required' => TRUE,
  ];

  $form['remote_url'] = [
    '#type' => 'textfield',
    '#title' => t('Gateway URL'),
    '#description' => t('The Global Payments URL to submit the order to.'),
    '#default_value' => $settings['remote_url'] ?? '',  // https://hpp.sandbox.realexpayments.com/pay
    '#size' => 50,
    '#required' => TRUE,
  ];

  $form['auto_settle'] = [
    '#type' => 'textfield',
    '#title' => t('AutoSettle'),
    '#default_value' => $settings['auto_settle'] ?? '1',
    '#description' => t('AutoSettle flag. Allowed values are 0, 1 or MULTI.'),
  ];

  return $form;
}

/**
 * Implements hook_commerce_payment_method_submit_form().
 *
 * Set allowed credit card types.
 */
function global_payments_administrator_terminal_commerce_payment_method_submit_form($payment_method, $pane_values, $checkout_pane, $order) {
  $order_wrapper = entity_metadata_wrapper('commerce_order', $order);
  $profile = $order_wrapper->value();
  $profile_billing = $order_wrapper->commerce_customer_billing->value();
  $profile_shipping = $order_wrapper->commerce_customer_shipping->value();

  $customer_billing_address = $profile_billing->commerce_customer_address[LANGUAGE_NONE][0];
  $billing_country = $customer_billing_address['country'];
  $billing_city = $customer_billing_address['locality'];
  $billing_address_1 = $customer_billing_address['thoroughfare'];
  $billing_address_2 = $customer_billing_address['premise'];
  $billing_postal_code = $customer_billing_address['postal_code'];

  $customer_shipping_address = $profile_shipping->commerce_customer_address[LANGUAGE_NONE][0];
  $shipping_country = $customer_shipping_address['country'];
  $shipping_city = $customer_shipping_address['locality'];
  $shipping_address_1 = $customer_shipping_address['thoroughfare'];
  $shipping_address_2 = $customer_shipping_address['premise'];
  $shipping_postal_code = $customer_shipping_address['postal_code'];

  $email = $profile->mail;
  $currency_code = $order->commerce_order_total["und"][0]["currency_code"];

  $commerce_realex_redirect_inc = drupal_get_path('module', 'commerce_realex') . '/includes/commerce_realex_redirect.inc';
  $hpp_address_match_indicator = array_values($customer_billing_address) == array_values($customer_shipping_address) ? 'TRUE' : 'FALSE';
  include_once $commerce_realex_redirect_inc;
  $form = [];
  // TODO: sanity check values?
  $data = [
    'MERCHANT_RESPONSE_URL' => url('commerce-realex/3dsecure2/terminal/complete', ['absolute' => TRUE]),
    'HPP_VERSION' => 2,
    'AUTO_SETTLE_FLAG' => $payment_method['settings']['auto_settle'] ?? 1,
    'MERCHANT_ID' => $payment_method['settings']['merchant_id'],
    'ORDER_ID' => $order->order_id . '-' . strftime('%Y%m%d%H%M%S'),
    'ACCOUNT' => $payment_method["settings"]["account_id"],
    'COMMERCE_ORDER_ID' => $order->order_id,
    'COMMERCE_ORDER_NUMBER' => $order->order_number,
    'RETURN_TSS' => 1,
    'HPP_ADDRESS_MATCH_INDICATOR' => $hpp_address_match_indicator,

    'HPP_BILLING_STREET1' => $billing_address_1,
    'HPP_BILLING_STREET2' => $billing_address_2,
    'HPP_BILLING_CITY' => $billing_city,
    'HPP_BILLING_POSTALCODE' => $billing_postal_code,
    'HPP_BILLING_COUNTRY' => commerce_realex_get_country_numeric_code($billing_country),

    'HPP_SHIPPING_STREET1' => $shipping_address_1,
    'HPP_SHIPPING_STREET2' => $shipping_address_2,
    'HPP_SHIPPING_CITY' => $shipping_city,
    'HPP_SHIPPING_POSTALCODE' => $shipping_postal_code,
    'HPP_SHIPPING_COUNTRY' => commerce_realex_get_country_numeric_code($shipping_country),

    'currency_code' => $currency_code,
    'remote_url' => $payment_method['settings']['remote_url'],
    'shared_secret' => $payment_method['settings']['shared_secret'],
  ];
  $form['HPP_CUSTOMER_EMAIL'] = [
    '#title' => t('Customer email'),
    '#type' => 'textfield',
    '#required' => 1,
    '#default_value' => !empty($payment_method['settings']['billing_email']) ? $payment_method['settings']['billing_email'] : '',
  ];
  foreach ($data as $name => $value) {
    if (!empty($value)) {
      $form[$name] = ['#type' => 'hidden', '#value' => $value];
    }
  }
  return $form;
}

/**
 * Prepare the form to be Realex compatible.
 *
 * This is nowhere land; form not Drpal managed that sends to Realex.
 *
 * @return string
 *   It returns a form with action = endpoint.
 */
function global_payments_administrator_terminal_checkout_prepare() {
  // TODO: sanity check values?
  $post_vars = drupal_get_query_parameters($_POST);
  $timestamp = strftime('%Y%m%d%H%M%S');
  mt_srand((double) microtime() * 1000000);

  $request['timestamp'] = $timestamp;
  $request['orderid'] = $post_vars['payment_details']['ORDER_ID'];
  $request['merchantid'] = check_plain($post_vars['payment_details']['MERCHANT_ID']);
  $request['amount'] = number_format((float) $post_vars['amount'], 2, '.', '') * 100;
  $request['currency'] = $post_vars['payment_details']['currency_code'];
  $request['secret'] = check_plain($post_vars['payment_details']['shared_secret']);
  unset($post_vars['payment_details']['shared_secret']);
  $sha1hash = _commerce_realex_build_request_sha1hash($request, 'redirect-auth');
  $form = [
    '#type' => 'form',
    '#action' => $post_vars["payment_details"]['remote_url'],
    '#id' => 'global_payments_administrator_terminal_checkout_prepare',
  ];
  $form['Prepare payment'] = [
    '#markup' => '<h1>' . t('Proceed to the payment.') .
    '</h1><p>' . t('You will be redirected to the payment page.') . '</p>',
  ];

  $form['AMOUNT'] = [
    '#type' => 'hidden',
    '#value' => $request['amount'],
    '#name' => 'AMOUNT',
  ];

  $form['CURRENCY_CODE'] = [
    '#type' => 'hidden',
    '#value' => $post_vars['currency_code'],
    '#name' => 'CURRENCY',
  ];

  $form['TIMESTAMP'] = [
    '#type' => 'hidden',
    '#value' => $timestamp,
    '#name' => 'TIMESTAMP',
  ];

  $form['SHA1HASH'] = [
    '#type' => 'hidden',
    '#value' => $sha1hash,
    '#name' => 'SHA1HASH',
  ];

  foreach ($post_vars["payment_details"] as $name => $value) {
    if (!empty($value)) {
      $form[$name] = [
        '#type' => 'hidden',
        '#value' => $value,
        '#name' => $name,
      ];
    }
  }
  $form['submit'] = [
    '#type' => 'submit',
    '#value' => 'Proceed to payment now',
  ];

  return drupal_render($form);
}

/**
 * Callback url for Realex.
 */
function global_payments_administrator_terminal_checkout_complete() {
  // Pull out the response data.
  $response = [];
  foreach ($_POST as $key => $value) {
    // Modify keys to lower case.
    $key = strtolower($key);
    $response[$key] = $value;
  }
  if (_global_payments_administrator_terminal_validate_callback($response)) {

    if (!empty($response['commerce_order_id'])) {
      $order_id = $response['commerce_order_id'];
      $success = $response['result'] == '00';
      if ($success) {
        global $base_url;
        $payment_method = commerce_payment_method_load(GLOBAL_PAYMENTS_ADMINISTRATOR_TERMINAL_PAYMENT_METHOD_ID);
        $transaction = commerce_payment_transaction_new(GLOBAL_PAYMENTS_ADMINISTRATOR_TERMINAL_PAYMENT_METHOD_ID, $order_id);
        $charge = [];
        $charge['amount'] = $response['amount'];
        $charge['currency_code'] = $response['currency_code'];

        array_walk($response, function (&$item, $key) {
          $item = $key . ": " . $item;
        });
        $message = implode('<br>', $response);
        global_payments_administrator_terminal_transaction($transaction, $payment_method, $charge, $message);
        $order_url = $base_url . "/admin/commerce/orders/$order_id/payment";
        watchdog('global_payments_administrator_terminal', 'Payment done: %message', ['%message' => $response['message']], WATCHDOG_INFO);
        return _global_payments_administrator_terminal_redirect_back($order_url);
      }
    }
  }
  else {
    watchdog('global_payments_administrator_terminal', 'Possible attepmt of fraudulent payment callback invokation');
    return t('Payment not completed. Could not establish the authenticity of the caller.');
  }

  // Invalid response or failure.
  $message = t('We were unable to process your credit card payment; reason provided by bank: %message. If the problem persists, contact us to complete your order.', ['%message' => $response['message']]);
  watchdog('global_payments_administrator_terminal', 'Payment failed: %message', ['%message' => $response['message']], WATCHDOG_WARNING);
  return $message;

}

/**
 * Save a transaction.
 */
function global_payments_administrator_terminal_transaction($transaction, $payment_method, $charge, $message) {
  $transaction->instance_id = $payment_method['instance_id'];
  $transaction->amount = $charge['amount'];
  $transaction->currency_code = $charge['currency_code'];
  $transaction->status = COMMERCE_PAYMENT_STATUS_SUCCESS;
  $transaction->message = $message;

  commerce_payment_transaction_save($transaction);
}

/**
 * Redirect back to given url on the website.
 */
function _global_payments_administrator_terminal_redirect_back($url) {
  $output = '<style type="text/css">body {display: none;}</style>';
  $output .= '<script type="text/javascript"> window.location = "' . $url . '";</script>';
  print $output;
  return '';
}

/**
 * A bit of validation for the callback.
 */
function _global_payments_administrator_terminal_validate_callback($response) {
  if (
    !empty($response['result']) &&
    !empty($response['commerce_order_id']) &&
    !empty($response['remote_url']) &&
    !empty($response['timestamp']) &&
    !empty($response['sha1hash'])
  ) {
    // TODO: add additional check?
    return TRUE;
  }
  else {
    return FALSE;
  }
}
